@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # input-remapper - GUI for device specific keyboard mappings
-# Copyright (C) 2023 sezanzeb <proxima@sezanzeb.de>
+# Copyright (C) 2025 sezanzeb <b8x45ygc9@mozmail.com>
 #
 # This file is part of input-remapper.
 #
@@ -20,10 +20,10 @@
 from __future__ import annotations
 
 import itertools
-from typing import Tuple, Iterable, Union, List, Dict, Optional, Hashable, NewType
+from typing import Tuple, Iterable, Union, List, Dict, Optional, Hashable
 
 from evdev import ecodes
-
+from inputremapper.configs.paths import PathUtils
 from inputremapper.input_event import InputEvent
 
 try:
@@ -31,10 +31,10 @@ try:
 except ImportError:
     from pydantic import BaseModel, root_validator, validator
 
-from inputremapper.configs.system_mapping import system_mapping
+from inputremapper.configs.keyboard_layout import keyboard_layout
 from inputremapper.gui.messages.message_types import MessageType
-from inputremapper.logger import logger
-from inputremapper.utils import get_evdev_constant_name
+from inputremapper.logging.logger import logger
+from inputremapper.utils import get_evdev_constant_name, DeviceHash
 
 # having shift in combinations modifies the configured output,
 # ctrl might not work at all
@@ -47,8 +47,6 @@ DIFFICULT_COMBINATIONS = [
     ecodes.KEY_RIGHTALT,
 ]
 
-DeviceHash = NewType("DeviceHash", str)
-
 EMPTY_TYPE = 99
 
 
@@ -142,15 +140,13 @@ class InputConfig(BaseModel):
         # first try to find the name in xmodmap to not display wrong
         # names due to the keyboard layout
         if self.type == ecodes.EV_KEY:
-            key_name = system_mapping.get_name(self.code)
+            key_name = keyboard_layout.get_name(self.code)
 
         if key_name is None:
             # if no result, look in the linux combination constants. On a german
             # keyboard for example z and y are switched, which will therefore
             # cause the wrong letter to be displayed.
             key_name = get_evdev_constant_name(self.type, self.code)
-            if isinstance(key_name, list):
-                key_name = key_name[0]
 
         key_name = key_name.replace("ABS_Z", "Trigger Left")
         key_name = key_name.replace("ABS_RZ", "Trigger Right")
@@ -238,9 +234,11 @@ class InputConfig(BaseModel):
             type=type_ if type_ is not None else self.type,
             code=code if code is not None else self.code,
             origin_hash=origin_hash if origin_hash is not None else self.origin_hash,
-            analog_threshold=analog_threshold
-            if analog_threshold is not None
-            else self.analog_threshold,
+            analog_threshold=(
+                analog_threshold
+                if analog_threshold is not None
+                else self.analog_threshold
+            ),
         )
 
     def __hash__(self):
@@ -421,6 +419,14 @@ class InputCombination(Tuple[InputConfig, ...]):
         if len(self) <= 2:
             return [self]
 
+        if len(self) > 6:
+            logger.warning(
+                "Your input combination has a length of %d. Long combinations might "
+                'freeze the process. Edit the configuration files in "%s" to fix it.',
+                len(self),
+                PathUtils.get_config_path(),
+            )
+
         permutations = []
         for permutation in itertools.permutations(self[:-1]):
             permutations.append(InputCombination((*permutation, self[-1])))
