@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # input-remapper - GUI for device specific keyboard mappings
-# Copyright (C) 2023 sezanzeb <proxima@sezanzeb.de>
+# Copyright (C) 2025 sezanzeb <b8x45ygc9@mozmail.com>
 #
 # This file is part of input-remapper.
 #
@@ -37,11 +37,9 @@ import multiprocessing
 import os
 import re
 import threading
-import traceback
 from typing import List, Optional
 
 import evdev
-from evdev import InputDevice
 from evdev.ecodes import (
     EV_KEY,
     EV_ABS,
@@ -56,8 +54,8 @@ from evdev.ecodes import (
     REL_WHEEL,
 )
 
-from inputremapper.configs.paths import get_preset_path
-from inputremapper.logger import logger
+from inputremapper.configs.paths import PathUtils
+from inputremapper.logging.logger import logger
 from inputremapper.utils import get_device_hash
 
 TABLET_KEYS = [
@@ -197,7 +195,7 @@ def classify(device) -> DeviceType:
 DENYLIST = [".*Yubico.*YubiKey.*", "Eee PC WMI hotkeys"]
 
 
-def is_denylisted(device: InputDevice):
+def is_denylisted(device: evdev.InputDevice):
     """Check if a device should not be used in input-remapper.
 
     Parameters
@@ -211,7 +209,7 @@ def is_denylisted(device: InputDevice):
     return False
 
 
-def get_unique_key(device: InputDevice):
+def get_unique_key(device: evdev.InputDevice):
     """Find a string key that is unique for a single hardware device.
 
     All InputDevices in /dev/input that originate from the same physical
@@ -221,8 +219,6 @@ def get_unique_key(device: InputDevice):
     # - device.phys is empty sometimes and varies across virtual
     #   subdevices
     # - device.version varies across subdevices
-    # - device.uniq is empty most of the time, I don't know what this is
-    #   supposed to be
     return (
         # device.info bustype, vendor and product are unique for
         # a product, but multiple similar device models would be grouped
@@ -230,6 +226,9 @@ def get_unique_key(device: InputDevice):
         f"{device.info.bustype}_"
         f"{device.info.vendor}_"
         f"{device.info.product}_"
+        # device.uniq is empty most of the time. It seems to be the only way to
+        # distinguish multiple connected bluetooth gamepads
+        f"{device.uniq}_"
         # deivce.phys if "/input..." is removed from it, because the first
         # chunk seems to be unique per hardware (if it's not completely empty)
         f'{device.phys.split("/")[0] or "-"}'
@@ -298,7 +297,7 @@ class _Group:
         This path is unique per device-model, not per group. Groups
         of the same model share the same preset paths.
         """
-        return get_preset_path(self.name, preset)
+        return PathUtils.get_preset_path(self.name, preset)
 
     def get_devices(self) -> List[evdev.InputDevice]:
         devices: List[evdev.InputDevice] = []
@@ -385,9 +384,11 @@ class _FindGroups(threading.Thread):
             capabilities = device.capabilities(absinfo=False)
 
             key_capa = capabilities.get(EV_KEY)
+            abs_capa = capabilities.get(EV_ABS)
+            rel_capa = capabilities.get(EV_REL)
 
-            if key_capa is None and device_type != DeviceType.GAMEPAD:
-                # skip devices that don't provide buttons that can be mapped
+            if key_capa is None and abs_capa is None and rel_capa is None:
+                # skip devices that don't provide buttons or axes that can be mapped
                 logger.debug('"%s" has no useful capabilities', device.name)
                 continue
 
@@ -475,7 +476,7 @@ class _Groups:
         self.loads(pipe[0].recv())
 
         if len(self._groups) == 0:
-            logger.debug("Did not find any input device")
+            logger.error("Did not find any input device")
         else:
             keys = [f'"{group.key}"' for group in self._groups]
             logger.info("Found %s", ", ".join(keys))
