@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # input-remapper - GUI for device specific keyboard mappings
-# Copyright (C) 2023 sezanzeb <proxima@sezanzeb.de>
+# Copyright (C) 2025 sezanzeb <b8x45ygc9@mozmail.com>
 #
 # This file is part of input-remapper.
 #
@@ -27,18 +27,18 @@ from typing import List, Dict, Set, Hashable
 
 import evdev
 
-from inputremapper.configs.input_config import DeviceHash
-from inputremapper.input_event import InputEvent
 from inputremapper.configs.preset import Preset
 from inputremapper.injection.mapping_handlers.mapping_handler import (
     EventListener,
     NotifyCallback,
 )
 from inputremapper.injection.mapping_handlers.mapping_parser import (
-    parse_mappings,
+    MappingParser,
     EventPipelines,
 )
-from inputremapper.logger import logger
+from inputremapper.input_event import InputEvent
+from inputremapper.logging.logger import logger
+from inputremapper.utils import DeviceHash
 
 
 class Context:
@@ -82,18 +82,19 @@ class Context:
         preset: Preset,
         source_devices: Dict[DeviceHash, evdev.InputDevice],
         forward_devices: Dict[DeviceHash, evdev.UInput],
+        mapping_parser: MappingParser,
     ):
         if len(forward_devices) == 0:
-            logger.warning("Not forward_devices set")
+            logger.warning("forward_devices not set")
 
         if len(source_devices) == 0:
-            logger.warning("Not source_devices set")
+            logger.warning("source_devices not set")
 
         self.listeners = set()
         self._source_devices = source_devices
         self._forward_devices = forward_devices
         self._notify_callbacks = defaultdict(list)
-        self._handlers = parse_mappings(preset, self)
+        self._handlers = mapping_parser.parse_mappings(preset, self)
 
         self._create_callbacks()
 
@@ -122,3 +123,10 @@ class Context:
 
     def get_source(self, key: DeviceHash) -> evdev.InputDevice:
         return self._source_devices[key]
+
+    def get_leds(self) -> Set[int]:
+        """Get a set of LED_* ecodes that are currently on."""
+        leds = set()
+        for device in self._source_devices.values():
+            leds.update(device.leds())
+        return leds
