@@ -1,7 +1,7 @@
-#!/usr/bin/python3
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 # input-remapper - GUI for device specific keyboard mappings
-# Copyright (C) 2023 sezanzeb <proxima@sezanzeb.de>
+# Copyright (C) 2025 sezanzeb <b8x45ygc9@mozmail.com>
 #
 # This file is part of input-remapper.
 #
@@ -19,32 +19,17 @@
 # along with input-remapper.  If not, see <https://www.gnu.org/licenses/>.
 
 import asyncio
-
-# the tests file needs to be imported first to make sure patches are loaded
+import atexit
+import multiprocessing
+import os
+import time
+import unittest
 from contextlib import contextmanager
 from typing import Tuple, List, Optional, Iterable
+from unittest.mock import patch, MagicMock, call
 
-from inputremapper.gui.autocompletion import get_incomplete_parameter, _get_left_text
-
-from inputremapper.injection.global_uinputs import global_uinputs
-from tests.lib.global_uinputs import reset_global_uinputs_for_service
-from tests.test import get_project_root
-from tests.lib.cleanup import cleanup, quick_cleanup
-from tests.lib.stuff import spy
-from tests.lib.constants import EVENT_READ_TIMEOUT
-from tests.lib.fixtures import prepare_presets
-from tests.lib.logger import logger
-from tests.lib.fixtures import fixtures
-from tests.lib.pipes import push_event, push_events, uinput_write_history_pipe
-from tests.integration.test_components import FlowBoxTestUtils
-
-import sys
-import time
-import atexit
-import os
-import unittest
-import multiprocessing
 import evdev
+import gi
 from evdev.ecodes import (
     EV_KEY,
     EV_ABS,
@@ -53,13 +38,22 @@ from evdev.ecodes import (
     KEY_Q,
     EV_REL,
 )
-from unittest.mock import patch, MagicMock, call
-from importlib.util import spec_from_loader, module_from_spec
-from importlib.machinery import SourceFileLoader
 
+from inputremapper.gui.autocompletion import (
+    get_incomplete_parameter,
+    get_incomplete_function_name,
+)
+from inputremapper.injection.global_uinputs import GlobalUInputs, FrontendUInput, UInput
+from inputremapper.injection.mapping_handlers.mapping_parser import MappingParser
 from inputremapper.input_event import InputEvent
-
-import gi
+from tests.integration.test_components import FlowBoxTestUtils
+from tests.lib.cleanup import cleanup
+from tests.lib.constants import EVENT_READ_TIMEOUT
+from tests.lib.fixtures import fixtures
+from tests.lib.fixtures import prepare_presets
+from tests.lib.logger import logger
+from tests.lib.pipes import push_event, push_events, uinput_write_history_pipe
+from tests.lib.spy import spy
 
 gi.require_version("Gdk", "3.0")
 gi.require_version("Gtk", "3.0")
@@ -67,10 +61,10 @@ gi.require_version("GtkSource", "4")
 gi.require_version("GLib", "2.0")
 from gi.repository import Gtk, GLib, Gdk, GtkSource
 
-from inputremapper.configs.system_mapping import system_mapping
+from inputremapper.configs.keyboard_layout import keyboard_layout
 from inputremapper.configs.mapping import Mapping
-from inputremapper.configs.paths import CONFIG_PATH, get_preset_path, get_config_path
-from inputremapper.configs.global_config import global_config
+from inputremapper.configs.paths import PathUtils
+from inputremapper.configs.global_config import GlobalConfig
 from inputremapper.groups import _Groups
 from inputremapper.gui.data_manager import DataManager
 from inputremapper.gui.messages.message_broker import (
@@ -78,7 +72,11 @@ from inputremapper.gui.messages.message_broker import (
     MessageType,
 )
 from inputremapper.gui.messages.message_data import StatusData, CombinationRecorded
-from inputremapper.gui.components.editor import MappingSelectionLabel, SET_KEY_FIRST
+from inputremapper.gui.components.editor import (
+    MappingSelectionLabel,
+    SET_KEY_FIRST,
+    CodeEditor,
+)
 from inputremapper.gui.components.device_groups import DeviceGroupEntry
 from inputremapper.gui.controller import Controller
 from inputremapper.gui.reader_service import ReaderService
@@ -87,6 +85,9 @@ from inputremapper.gui.user_interface import UserInterface
 from inputremapper.injection.injector import InjectorState
 from inputremapper.configs.input_config import InputCombination, InputConfig
 from inputremapper.daemon import Daemon, DaemonProxy
+from inputremapper.bin.input_remapper_gtk import InputRemapperGtkBin
+
+from tests.lib.test_setup import test_setup
 
 
 # iterate a few times when Gtk.main() is called, but don't block
@@ -98,70 +99,71 @@ Gtk.main = gtk_iteration
 Gtk.main_quit = lambda: None
 
 
-def launch(
-    argv=None,
-) -> Tuple[UserInterface, Controller, DataManager, MessageBroker, DaemonProxy]:
-    """Start input-remapper-gtk with the command line argument array argv."""
-    bin_path = os.path.join(get_project_root(), "bin", "input-remapper-gtk")
-
-    if not argv:
-        argv = ["-d"]
-
-    with patch.object(sys, "argv", [""] + [str(arg) for arg in argv]):
-        loader = SourceFileLoader("__main__", bin_path)
-        spec = spec_from_loader("__main__", loader)
-        module = module_from_spec(spec)
-        spec.loader.exec_module(module)
-
+def launch() -> Tuple[
+    UserInterface,
+    Controller,
+    DataManager,
+    MessageBroker,
+    DaemonProxy,
+    GlobalConfig,
+]:
+    """Start input-remapper-gtk."""
+    return_ = InputRemapperGtkBin.main()
     gtk_iteration()
-
     # otherwise a new handler is added with each call to launch, which
     # spams tons of garbage when all tests finish
-    atexit.unregister(module.stop)
-
-    return (
-        module.user_interface,
-        module.controller,
-        module.data_manager,
-        module.message_broker,
-        module.daemon,
-    )
+    atexit.unregister(InputRemapperGtkBin.stop)
+    return return_
 
 
 def start_reader_service():
     def process():
-        reader_service = ReaderService(_Groups())
+        global_uinputs = GlobalUInputs(FrontendUInput)
+        reader_service = ReaderService(_Groups(), global_uinputs)
         loop = asyncio.new_event_loop()
         loop.run_until_complete(reader_service.run())
 
     multiprocessing.Process(target=process).start()
 
 
+def os_system_patch(cmd, original_os_system=os.system):
+    # instead of running pkexec, fork instead. This will make
+    # the reader-service aware of all the test patches
+    if "pkexec input-remapper-control --command start-reader-service" in cmd:
+        logger.info("pkexec-patch starting ReaderService process")
+        start_reader_service()
+        return 0
+
+    return original_os_system(cmd)
+
+
 @contextmanager
-def patch_launch():
-    """patch the launch function such that we don't connect to
-    the dbus and don't use pkexec to start the reader-service"""
-    original_connect = Daemon.connect
-    original_os_system = os.system
-    Daemon.connect = Daemon
+def patch_services():
+    """Don't connect to the dbus and don't use pkexec to start the reader-service"""
 
-    def os_system(cmd):
-        # instead of running pkexec, fork instead. This will make
-        # the reader-service aware of all the test patches
-        if "pkexec input-remapper-control --command start-reader-service" in cmd:
-            logger.info("pkexec-patch starting ReaderService process")
-            start_reader_service()
-            return 0
+    def bootstrap_daemon():
+        # The daemon gets fresh instances of everything, because as far as I remember
+        # it runs in a separate process.
+        global_config = GlobalConfig()
+        global_uinputs = GlobalUInputs(UInput)
+        mapping_parser = MappingParser(global_uinputs)
 
-        return original_os_system(cmd)
+        return Daemon(
+            global_config,
+            global_uinputs,
+            mapping_parser,
+        )
 
-    os.system = os_system
-    yield
-    os.system = original_os_system
-    Daemon.connect = original_connect
+    with patch.object(
+        os,
+        "system",
+        os_system_patch,
+    ), patch.object(Daemon, "connect", bootstrap_daemon):
+        yield
 
 
 def clean_up_integration(test):
+    logger.info("clean_up_integration")
     test.controller.stop_injecting()
     gtk_iteration()
     test.user_interface.on_gtk_close()
@@ -183,19 +185,10 @@ class GtkKeyEvent:
         return True, self.keyval
 
 
+@test_setup
 class TestGroupsFromReaderService(unittest.TestCase):
-    def setUp(self):
-        # don't try to connect, return an object instance of it instead
-        self.original_connect = Daemon.connect
-        Daemon.connect = Daemon
-
-        # this is already part of the test. we need a bit of patching and hacking
-        # because we want to discover the groups as early a possible, to reduce startup
-        # time for the application
-        self.original_os_system = os.system
-        self.reader_service_started = MagicMock()
-
-        def os_system(cmd):
+    def patch_os_system(self):
+        def os_system(cmd, original_os_system=os.system):
             # instead of running pkexec, fork instead. This will make
             # the reader-service aware of all the test patches
             if "pkexec input-remapper-control --command start-reader-service" in cmd:
@@ -203,21 +196,59 @@ class TestGroupsFromReaderService(unittest.TestCase):
                 self.reader_service_started()
                 return 0
 
-            return self.original_os_system(cmd)
+            return original_os_system(cmd)
+
+        self.os_system_patch = patch.object(
+            os,
+            "system",
+            os_system,
+        )
+
+        # this is already part of the test. we need a bit of patching and hacking
+        # because we want to discover the groups as early a possible, to reduce startup
+        # time for the application
+        self.os_system_patch.start()
+
+    def bootstrap_daemon(self):
+        # The daemon gets fresh instances of everything, because as far as I remember
+        # it runs in a separate process.
+        global_config = GlobalConfig()
+        global_uinputs = GlobalUInputs(UInput)
+        mapping_parser = MappingParser(global_uinputs)
+
+        return Daemon(
+            global_config,
+            global_uinputs,
+            mapping_parser,
+        )
+
+    def patch_daemon(self):
+        # don't try to connect, return an object instance of it instead
+        self.daemon_connect_patch = patch.object(
+            Daemon,
+            "connect",
+            lambda: self.bootstrap_daemon(),
+        )
+        self.daemon_connect_patch.start()
+
+    def setUp(self):
+        self.reader_service_started = MagicMock()
+        self.patch_os_system()
+        self.patch_daemon()
 
-        os.system = os_system
         (
             self.user_interface,
             self.controller,
             self.data_manager,
             self.message_broker,
             self.daemon,
+            self.global_config,
         ) = launch()
 
     def tearDown(self):
         clean_up_integration(self)
-        os.system = self.original_os_system
-        Daemon.connect = self.original_connect
+        self.os_system_patch.stop()
+        self.daemon_connect_patch.stop()
 
     def test_knows_devices(self):
         # verify that it is working as expected. The gui doesn't have knowledge
@@ -242,49 +273,50 @@ class TestGroupsFromReaderService(unittest.TestCase):
         self.assertEqual(self.data_manager.active_group.name, "Foo Device")
 
 
-class PatchedConfirmDelete:
-    def __init__(self, user_interface: UserInterface, response=Gtk.ResponseType.ACCEPT):
-        self.response = response
-        self.user_interface = user_interface
-        self._original_create_dialog = user_interface._create_dialog
-        self.patch = None
+@contextmanager
+def patch_confirm_delete(
+    user_interface: UserInterface,
+    response=Gtk.ResponseType.ACCEPT,
+):
+    original_create_dialog = user_interface._create_dialog
 
-    def _create_dialog_patch(self, *args, **kwargs):
+    def _create_dialog_patch(*args, **kwargs):
         """A patch for the deletion confirmation that briefly shows the dialog."""
-        confirm_cancel_dialog = self._original_create_dialog(*args, **kwargs)
+        confirm_cancel_dialog = original_create_dialog(*args, **kwargs)
+
         # the emitted signal causes the dialog to close
         GLib.timeout_add(
             100,
-            lambda: confirm_cancel_dialog.emit("response", self.response),
+            lambda: confirm_cancel_dialog.emit("response", response),
         )
-        Gtk.MessageDialog.run(confirm_cancel_dialog)  # don't recursively call the patch
 
-        confirm_cancel_dialog.run = lambda: self.response
+        # don't recursively call the patch
+        Gtk.MessageDialog.run(confirm_cancel_dialog)
+
+        confirm_cancel_dialog.run = lambda: response
 
         return confirm_cancel_dialog
 
-    def __enter__(self):
-        self.patch = patch.object(
-            self.user_interface,
-            "_create_dialog",
-            self._create_dialog_patch,
-        )
-        self.patch.__enter__()
-
-    def __exit__(self, *args, **kwargs):
-        self.patch.__exit__(*args, **kwargs)
+    with patch.object(
+        user_interface,
+        "_create_dialog",
+        _create_dialog_patch,
+    ):
+        # Tests are run during `yield`
+        yield
 
 
 class GuiTestBase(unittest.TestCase):
     def setUp(self):
         prepare_presets()
-        with patch_launch():
+        with patch_services():
             (
                 self.user_interface,
                 self.controller,
                 self.data_manager,
                 self.message_broker,
                 self.daemon,
+                self.global_config,
             ) = launch()
 
         get = self.user_interface.get
@@ -319,7 +351,7 @@ class GuiTestBase(unittest.TestCase):
 
         evdev.InputDevice.grab = grab
 
-        global_config._save_config()
+        self.global_config._save_config()
 
         self.throttle(20)
 
@@ -400,6 +432,17 @@ class GuiTestBase(unittest.TestCase):
 
         self.throttle(20)
 
+    def focus_source_view(self):
+        # despite the focus and gtk_iterations, gtk never runs the event handlers for
+        # the focus-in-event (_update_placeholder), which would clear the placeholder
+        # text. Remove it manually, it can't be helped. Fun fact: when the
+        # window gets destroyed, gtk runs the handler 10 times for good measure.
+        # Lost one hour of my life on GTK again. It's gone! Forever! Use qt next time.
+        source_view = self.code_editor
+        self.set_focus(source_view)
+        self.code_editor.get_buffer().set_text("")
+        return source_view
+
     def get_selection_labels(self) -> List[MappingSelectionLabel]:
         return self.selection_label_listbox.get_children()
 
@@ -449,6 +492,7 @@ class GuiTestBase(unittest.TestCase):
         gtk_iteration()
 
 
+@test_setup
 class TestColors(GuiTestBase):
     # requires a running ui, otherwise fails with segmentation faults
     def test_get_color_falls_back(self):
@@ -470,10 +514,10 @@ class TestColors(GuiTestBase):
         )
 
         self.assertIsInstance(color, Gdk.RGBA)
-        self.assertNotAlmostEquals(color.red, fallback.red, delta=0.01)
-        self.assertNotAlmostEquals(color.green, fallback.blue, delta=0.01)
-        self.assertNotAlmostEquals(color.blue, fallback.green, delta=0.01)
-        self.assertNotAlmostEquals(color.alpha, fallback.alpha, delta=0.01)
+        self.assertNotAlmostEqual(color.red, fallback.red, delta=0.01)
+        self.assertNotAlmostEqual(color.green, fallback.blue, delta=0.01)
+        self.assertNotAlmostEqual(color.blue, fallback.green, delta=0.01)
+        self.assertNotAlmostEqual(color.alpha, fallback.alpha, delta=0.01)
 
     def _test_color_wont_fallback(self, get_color, fallback):
         color = get_color()
@@ -498,6 +542,7 @@ class TestColors(GuiTestBase):
         self._test_color_wont_fallback(Colors.get_font_color, Colors.fallback_font)
 
 
+@test_setup
 class TestGui(GuiTestBase):
     """For tests that use the window.
 
@@ -638,7 +683,7 @@ class TestGui(GuiTestBase):
         # it creates the file for that right away. It may have been possible
         # to write it such that it doesn't (its empty anyway), but it does,
         # so use that to test it in more detail.
-        path = get_preset_path("Bar Device", "new preset")
+        path = PathUtils.get_preset_path("Bar Device", "new preset")
         self.assertTrue(os.path.exists(path))
         with open(path, "r") as file:
             self.assertEqual(file.read(), "")
@@ -1379,7 +1424,7 @@ class TestGui(GuiTestBase):
         self.assertEqual(len(self.data_manager.active_preset), 2)
         self.assertEqual(len(self.selection_label_listbox.get_children()), 2)
 
-        with PatchedConfirmDelete(self.user_interface):
+        with patch_confirm_delete(self.user_interface):
             self.delete_mapping_btn.clicked()
             gtk_iteration()
 
@@ -1424,7 +1469,7 @@ class TestGui(GuiTestBase):
         self.rename_btn.clicked()
         gtk_iteration()
 
-        preset_path = f"{CONFIG_PATH}/presets/Foo Device/foo.json"
+        preset_path = f"{PathUtils.config_path()}/presets/Foo Device/foo.json"
         self.assertTrue(os.path.exists(preset_path))
         error_icon = self.user_interface.get("error_status_icon")
         self.assertFalse(error_icon.get_visible())
@@ -1438,46 +1483,62 @@ class TestGui(GuiTestBase):
         status = self.get_status_text()
         self.assertIn("Permission denied", status)
 
-        with PatchedConfirmDelete(self.user_interface):
+        with patch_confirm_delete(self.user_interface):
             self.delete_preset_btn.clicked()
             gtk_iteration()
         self.assertFalse(os.path.exists(preset_path))
 
     def test_check_for_unknown_symbols(self):
+        first_input = InputCombination([InputConfig(type=1, code=1)])
+        second_input = InputCombination([InputConfig(type=1, code=2)])
         status = self.user_interface.get("status_bar")
         error_icon = self.user_interface.get("error_status_icon")
         warning_icon = self.user_interface.get("warning_status_icon")
 
         self.controller.load_preset("preset1")
         self.throttle(20)
-        self.controller.load_mapping(InputCombination([InputConfig(type=1, code=1)]))
+
+        # Switch to the first mapping, and change it
+        self.controller.load_mapping(first_input)
         gtk_iteration()
         self.controller.update_mapping(output_symbol="foo")
         gtk_iteration()
-        self.controller.load_mapping(InputCombination([InputConfig(type=1, code=2)]))
+
+        # Switch to the second mapping, and change it
+        self.controller.load_mapping(second_input)
         gtk_iteration()
         self.controller.update_mapping(output_symbol="qux")
         gtk_iteration()
 
+        # The tooltip should show the error of the currently selected mapping
         tooltip = status.get_tooltip_text().lower()
         self.assertIn("qux", tooltip)
         self.assertTrue(error_icon.get_visible())
         self.assertFalse(warning_icon.get_visible())
 
-        # it will still save it though
-        with open(get_preset_path("Foo Device", "preset1")) as f:
+        # So switching to the other mapping changes the tooltip
+        self.controller.load_mapping(first_input)
+        gtk_iteration()
+        tooltip = status.get_tooltip_text().lower()
+        self.assertIn("foo", tooltip)
+
+        # It will still save it though
+        with open(PathUtils.get_preset_path("Foo Device", "preset1")) as f:
             content = f.read()
             self.assertIn("qux", content)
             self.assertIn("foo", content)
 
+        # Fix the current active mapping.
+        # It should show the error of the other mapping now.
         self.controller.update_mapping(output_symbol="a")
         gtk_iteration()
         tooltip = status.get_tooltip_text().lower()
-        self.assertIn("foo", tooltip)
+        self.assertIn("qux", tooltip)
         self.assertTrue(error_icon.get_visible())
         self.assertFalse(warning_icon.get_visible())
 
-        self.controller.load_mapping(InputCombination([InputConfig(type=1, code=1)]))
+        # Fix the other mapping as well. No tooltip should be shown afterward.
+        self.controller.load_mapping(second_input)
         gtk_iteration()
         self.controller.update_mapping(output_symbol="b")
         gtk_iteration()
@@ -1486,6 +1547,18 @@ class TestGui(GuiTestBase):
         self.assertFalse(error_icon.get_visible())
         self.assertFalse(warning_icon.get_visible())
 
+    def test_no_validation_tooltip_for_empty_mappings(self):
+        self.controller.load_preset("preset1")
+        self.throttle(20)
+
+        status = self.user_interface.get("status_bar")
+        self.assertIsNone(status.get_tooltip_text())
+
+        self.controller.create_mapping()
+        gtk_iteration()
+        self.assertTrue(self.controller.is_empty_mapping())
+        self.assertIsNone(status.get_tooltip_text())
+
     def test_check_macro_syntax(self):
         status = self.status_bar
         error_icon = self.user_interface.get("error_status_icon")
@@ -1657,7 +1730,7 @@ class TestGui(GuiTestBase):
             self.assertFalse(error_icon.get_visible())
             wait()
             text = self.get_status_text()
-            self.assertIn("not grabbed", text)
+            self.assertIn("Failed to apply preset", text)
             self.assertTrue(error_icon.get_visible())
             self.assertNotEqual(
                 self.daemon.get_state("Foo Device 2"), InjectorState.RUNNING
@@ -1747,11 +1820,6 @@ class TestGui(GuiTestBase):
         self.assertIn("Stop", text)
 
     def test_start_injecting(self):
-        # It's 2023 everyone! That means this test randomly stopped working because it
-        # used FrontendUInputs instead of regular UInputs. I guess a fucking ghost
-        # was fixing this for us during 2022, but it seems to have disappeared.
-        reset_global_uinputs_for_service()
-
         self.controller.load_group("Foo Device 2")
 
         with spy(self.daemon, "set_config_dir") as spy1:
@@ -1761,7 +1829,7 @@ class TestGui(GuiTestBase):
                 # correctly uses group.key, not group.name
                 spy2.assert_called_once_with("Foo Device 2", "preset3")
 
-            spy1.assert_called_once_with(get_config_path())
+            spy1.assert_called_once_with(PathUtils.get_config_path())
 
         for _ in range(10):
             time.sleep(0.1)
@@ -1806,8 +1874,6 @@ class TestGui(GuiTestBase):
             self.assertNotIn("input-remapper", device_group_entry.name)
 
     def test_stop_injecting(self):
-        reset_global_uinputs_for_service()
-
         self.controller.load_group("Foo Device 2")
         self.start_injector_btn.clicked()
         gtk_iteration()
@@ -1864,19 +1930,25 @@ class TestGui(GuiTestBase):
 
     def test_delete_preset(self):
         # as per test_initial_state we already have preset3 loaded
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset3")))
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset3"))
+        )
 
-        with PatchedConfirmDelete(self.user_interface, Gtk.ResponseType.CANCEL):
+        with patch_confirm_delete(self.user_interface, Gtk.ResponseType.CANCEL):
             self.delete_preset_btn.clicked()
             gtk_iteration()
-            self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset3")))
+            self.assertTrue(
+                os.path.exists(PathUtils.get_preset_path("Foo Device", "preset3"))
+            )
             self.assertEqual(self.data_manager.active_preset.name, "preset3")
             self.assertEqual(self.data_manager.active_group.name, "Foo Device")
 
-        with PatchedConfirmDelete(self.user_interface):
+        with patch_confirm_delete(self.user_interface):
             self.delete_preset_btn.clicked()
             gtk_iteration()
-            self.assertFalse(os.path.exists(get_preset_path("Foo Device", "preset3")))
+            self.assertFalse(
+                os.path.exists(PathUtils.get_preset_path("Foo Device", "preset3"))
+            )
             self.assertEqual(self.data_manager.active_preset.name, "preset2")
             self.assertEqual(self.data_manager.active_group.name, "Foo Device")
 
@@ -1895,7 +1967,7 @@ class TestGui(GuiTestBase):
         unknown_key = "key-1234"
         self.device_selection.insert(
             DeviceGroupEntry(self.message_broker, self.controller, None, unknown_key),
-            0
+            0,
             # 0, [unknown_key, None, "foo"]
         )
 
@@ -1952,7 +2024,7 @@ class TestGui(GuiTestBase):
         self.assertNotEqual(presets1, presets3)
 
     def test_delete_last_preset(self):
-        with PatchedConfirmDelete(self.user_interface):
+        with patch_confirm_delete(self.user_interface):
             # as per test_initial_state we already have preset3 loaded
             self.assertEqual(self.data_manager.active_preset.name, "preset3")
 
@@ -1965,14 +2037,14 @@ class TestGui(GuiTestBase):
             self.delete_preset_btn.clicked()
             # the ui should be clean
             self.assert_gui_clean()
-            device_path = f"{CONFIG_PATH}/presets/{self.data_manager.active_group.name}"
+            device_path = f"{PathUtils.config_path()}/presets/{self.data_manager.active_group.name}"
             self.assertTrue(os.path.exists(f"{device_path}/new preset.json"))
 
             self.delete_preset_btn.clicked()
             gtk_iteration()
             # deleting an empty preset als doesn't do weird stuff
             self.assert_gui_clean()
-            device_path = f"{CONFIG_PATH}/presets/{self.data_manager.active_group.name}"
+            device_path = f"{PathUtils.config_path()}/presets/{self.data_manager.active_group.name}"
             self.assertTrue(os.path.exists(f"{device_path}/new preset.json"))
 
     def test_enable_disable_output(self):
@@ -2003,11 +2075,13 @@ class TestGui(GuiTestBase):
         )
         self.throttle(100)  # give time for the input to arrive
 
-        self.assertEqual(self.get_unfiltered_symbol_input_text(), "")
+        self.assertEqual(
+            self.get_unfiltered_symbol_input_text(), CodeEditor.placeholder
+        )
         self.assertTrue(self.output_box.get_sensitive())
 
         # disable it by deleting the mapping
-        with PatchedConfirmDelete(self.user_interface):
+        with patch_confirm_delete(self.user_interface):
             self.delete_mapping_btn.clicked()
             gtk_iteration()
 
@@ -2015,6 +2089,7 @@ class TestGui(GuiTestBase):
         self.assertFalse(self.output_box.get_sensitive())
 
 
+@test_setup
 class TestAutocompletion(GuiTestBase):
     def press_key(self, keyval):
         event = Gdk.EventKey()
@@ -2041,11 +2116,28 @@ class TestAutocompletion(GuiTestBase):
         test("foo", "foo")
         test("bar + foo", "foo")
 
+    def test_get_incomplete_function_name(self):
+        def test(text, expected):
+            text_view = Gtk.TextView()
+            Gtk.TextView.do_insert_at_cursor(text_view, text)
+            text_iter = text_view.get_iter_at_location(0, 0)[1]
+            text_iter.set_offset(len(text))
+            self.assertEqual(get_incomplete_function_name(text_iter), expected)
+
+        test("bar().foo", "foo")
+        test("bar()\n.foo", "foo")
+        test("bar().\nfoo", "foo")
+        test("bar(\nfoo", "foo")
+        test("bar(\nqux=foo", "foo")
+        test("bar(KEY_A,\nfoo", "foo")
+        test("foo", "foo")
+
     def test_autocomplete_names(self):
         autocompletion = self.user_interface.autocompletion
 
         def setup(text):
             self.set_focus(self.code_editor)
+            self.code_editor.get_buffer().set_text("")
             Gtk.TextView.do_insert_at_cursor(self.code_editor, text)
             self.throttle(200)
             text_iter = self.code_editor.get_iter_at_location(0, 0)[1]
@@ -2064,12 +2156,13 @@ class TestAutocompletion(GuiTestBase):
         gtk_iteration()
 
         self.set_focus(self.code_editor)
+        self.code_editor.get_buffer().set_text("")
 
         complete_key_name = "Test_Foo_Bar"
 
-        system_mapping.clear()
-        system_mapping._set(complete_key_name, 1)
-        system_mapping._set("KEY_A", 30)  # we need this for the UIMapping to work
+        keyboard_layout.clear()
+        keyboard_layout._set(complete_key_name, 1)
+        keyboard_layout._set("KEY_A", 30)  # we need this for the UIMapping to work
 
         # it can autocomplete a combination inbetween other things
         incomplete = "qux_1\n +  + qux_2"
@@ -2111,8 +2204,7 @@ class TestAutocompletion(GuiTestBase):
         self.controller.update_mapping(output_symbol="")
         gtk_iteration()
 
-        source_view = self.code_editor
-        self.set_focus(source_view)
+        source_view = self.focus_source_view()
 
         incomplete = "key(KEY_A).\nepea"
         Gtk.TextView.do_insert_at_cursor(source_view, incomplete)
@@ -2134,8 +2226,7 @@ class TestAutocompletion(GuiTestBase):
         self.controller.update_mapping(output_symbol="")
         gtk_iteration()
 
-        source_view = self.code_editor
-        self.set_focus(source_view)
+        source_view = self.focus_source_view()
 
         Gtk.TextView.do_insert_at_cursor(source_view, "KEY_")
 
@@ -2156,8 +2247,7 @@ class TestAutocompletion(GuiTestBase):
     def test_writing_still_works(self):
         self.controller.update_mapping(output_symbol="")
         gtk_iteration()
-        source_view = self.code_editor
-        self.set_focus(source_view)
+        source_view = self.focus_source_view()
 
         Gtk.TextView.do_insert_at_cursor(source_view, "KEY_")
 
@@ -2186,8 +2276,7 @@ class TestAutocompletion(GuiTestBase):
     def test_cycling(self):
         self.controller.update_mapping(output_symbol="")
         gtk_iteration()
-        source_view = self.code_editor
-        self.set_focus(source_view)
+        source_view = self.focus_source_view()
 
         Gtk.TextView.do_insert_at_cursor(source_view, "KEY_")
 
@@ -2214,6 +2303,7 @@ class TestAutocompletion(GuiTestBase):
         )
 
 
+@test_setup
 class TestDebounce(unittest.TestCase):
     def test_debounce(self):
         calls = 0
