@@ -1,7 +1,7 @@
-#!/usr/bin/python3
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 # input-remapper - GUI for device specific keyboard mappings
-# Copyright (C) 2023 sezanzeb <proxima@sezanzeb.de>
+# Copyright (C) 2025 sezanzeb <b8x45ygc9@mozmail.com>
 #
 # This file is part of input-remapper.
 #
@@ -17,28 +17,18 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with input-remapper.  If not, see <https://www.gnu.org/licenses/>.
+from inputremapper.injection.global_uinputs import GlobalUInputs, UInput
+from inputremapper.injection.macros.parse import Parser
+from inputremapper.injection.mapping_handlers.mapping_parser import MappingParser
 
 try:
     from pydantic.v1 import ValidationError
 except ImportError:
     from pydantic import ValidationError
 
-from inputremapper.input_event import InputEvent
-from tests.lib.global_uinputs import (
-    reset_global_uinputs_for_service,
-    reset_global_uinputs_for_gui,
-)
-from tests.lib.patches import uinputs
-from tests.lib.cleanup import quick_cleanup
-from tests.lib.constants import EVENT_READ_TIMEOUT
-from tests.lib.fixtures import fixtures
-from tests.lib.pipes import uinput_write_history_pipe
-from tests.lib.pipes import read_write_history_pipe, push_events
-from tests.lib.fixtures import keyboard_keys
-
+import time
 import unittest
 from unittest import mock
-import time
 
 import evdev
 from evdev.ecodes import (
@@ -53,6 +43,16 @@ from evdev.ecodes import (
     ABS_VOLUME,
 )
 
+from inputremapper.configs.input_config import InputCombination, InputConfig
+from inputremapper.configs.mapping import Mapping
+from inputremapper.configs.preset import Preset
+from inputremapper.configs.keyboard_layout import (
+    keyboard_layout,
+    DISABLE_CODE,
+    DISABLE_NAME,
+)
+from inputremapper.groups import groups, classify, DeviceType
+from inputremapper.injection.context import Context
 from inputremapper.injection.injector import (
     Injector,
     is_in_capabilities,
@@ -60,17 +60,15 @@ from inputremapper.injection.injector import (
     get_udev_name,
 )
 from inputremapper.injection.numlock import is_numlock_on
-from inputremapper.configs.system_mapping import (
-    system_mapping,
-    DISABLE_CODE,
-    DISABLE_NAME,
-)
-from inputremapper.configs.preset import Preset
-from inputremapper.configs.mapping import Mapping
-from inputremapper.configs.input_config import InputCombination, InputConfig
-from inputremapper.injection.macros.parse import parse
-from inputremapper.injection.context import Context
-from inputremapper.groups import groups, classify, DeviceType
+from inputremapper.input_event import InputEvent
+
+from tests.lib.constants import EVENT_READ_TIMEOUT
+from tests.lib.fixtures import fixtures
+from tests.lib.fixtures import keyboard_keys
+from tests.lib.patches import uinputs
+from tests.lib.pipes import read_write_history_pipe, push_events
+from tests.lib.pipes import uinput_write_history_pipe
+from tests.lib.test_setup import test_setup
 
 
 def wait_for_uinput_write():
@@ -80,6 +78,7 @@ def wait_for_uinput_write():
     return float(time.time() - start)
 
 
+@test_setup
 class TestInjector(unittest.IsolatedAsyncioTestCase):
     new_gamepad_path = "/dev/input/event100"
 
@@ -87,11 +86,13 @@ class TestInjector(unittest.IsolatedAsyncioTestCase):
     def setUpClass(cls):
         cls.injector = None
         cls.grab = evdev.InputDevice.grab
-        quick_cleanup()
 
     def setUp(self):
         self.failed = 0
         self.make_it_fail = 2
+        self.global_uinputs = GlobalUInputs(UInput)
+        self.global_uinputs.prepare_all()
+        self.mapping_parser = MappingParser(self.global_uinputs)
 
         def grab_fail_twice(_):
             if self.failed < self.make_it_fail:
@@ -106,15 +107,13 @@ class TestInjector(unittest.IsolatedAsyncioTestCase):
             time.sleep(0.2)
             self.assertIn(
                 self.injector.get_state(),
-                (InjectorState.STOPPED, InjectorState.FAILED, InjectorState.NO_GRAB),
+                (InjectorState.STOPPED, InjectorState.ERROR, InjectorState.NO_GRAB),
             )
             self.injector = None
         evdev.InputDevice.grab = self.grab
 
-        quick_cleanup()
-
     def initialize_injector(self, group, preset: Preset):
-        self.injector = Injector(group, preset)
+        self.injector = Injector(group, preset, self.mapping_parser)
         self.injector._devices = self.injector.group.get_devices()
         self.injector._update_preset()
 
@@ -130,10 +129,14 @@ class TestInjector(unittest.IsolatedAsyncioTestCase):
             )
         )
 
-        self.injector = Injector(groups.find(key="Foo Device 2"), preset)
+        self.injector = Injector(
+            groups.find(key="Foo Device 2"),
+            preset,
+            self.mapping_parser,
+        )
         # this test needs to pass around all other constraints of
         # _grab_device
-        self.injector.context = Context(preset, {}, {})
+        self.injector.context = Context(preset, {}, {}, self.mapping_parser)
         device = self.injector._grab_device(evdev.InputDevice(path))
         gamepad = classify(device) == DeviceType.GAMEPAD
         self.assertFalse(gamepad)
@@ -152,9 +155,13 @@ class TestInjector(unittest.IsolatedAsyncioTestCase):
             )
         )
 
-        self.injector = Injector(groups.find(key="Foo Device 2"), preset)
+        self.injector = Injector(
+            groups.find(key="Foo Device 2"),
+            preset,
+            self.mapping_parser,
+        )
         path = "/dev/input/event10"
-        self.injector.context = Context(preset, {}, {})
+        self.injector.context = Context(preset, {}, {}, self.mapping_parser)
         device = self.injector._grab_device(evdev.InputDevice(path))
         self.assertIsNone(device)
         self.assertGreaterEqual(self.failed, 1)
@@ -189,7 +196,7 @@ class TestInjector(unittest.IsolatedAsyncioTestCase):
             ),
         )
         self.initialize_injector(groups.find(name="gamepad"), preset)
-        self.injector.context = Context(preset, {}, {})
+        self.injector.context = Context(preset, {}, {}, self.mapping_parser)
         self.injector.group.paths = [
             "/dev/input/event10",
             "/dev/input/event30",
@@ -216,7 +223,7 @@ class TestInjector(unittest.IsolatedAsyncioTestCase):
         )
 
         self.initialize_injector(groups.find(name="gamepad"), preset)
-        self.injector.context = Context(preset, {}, {})
+        self.injector.context = Context(preset, {}, {}, self.mapping_parser)
 
         path = "/dev/input/event30"
         devices = self.injector._grab_devices()
@@ -236,7 +243,7 @@ class TestInjector(unittest.IsolatedAsyncioTestCase):
             )
         )
         self.initialize_injector(groups.find(key="Foo Device 2"), preset)
-        self.injector.context = Context(preset, {}, {})
+        self.injector.context = Context(preset, {}, {}, self.mapping_parser)
 
         # grabs only one device even though the group has 4 devices
         devices = self.injector._grab_devices()
@@ -255,7 +262,7 @@ class TestInjector(unittest.IsolatedAsyncioTestCase):
 
         # skips a device because its capabilities are not used in the preset
         self.initialize_injector(groups.find(key="Foo Device 2"), preset)
-        self.injector.context = Context(preset, {}, {})
+        self.injector.context = Context(preset, {}, {}, self.mapping_parser)
         devices = self.injector._grab_devices()
 
         # skips the device alltogether, so no grab attempts fail
@@ -263,7 +270,11 @@ class TestInjector(unittest.IsolatedAsyncioTestCase):
         self.assertEqual(devices, {})
 
     def test_get_udev_name(self):
-        self.injector = Injector(groups.find(key="Foo Device 2"), Preset())
+        self.injector = Injector(
+            groups.find(key="Foo Device 2"),
+            Preset(),
+            self.mapping_parser,
+        )
         suffix = "mapped"
         prefix = "input-remapper"
         expected = f'{prefix} {"a" * (80 - len(suffix) - len(prefix) - 2)} {suffix}'
@@ -308,7 +319,11 @@ class TestInjector(unittest.IsolatedAsyncioTestCase):
         )
         preset.add(m1)
         preset.add(m2)
-        self.injector = Injector(groups.find(key="Foo Device 2"), preset)
+        self.injector = Injector(
+            groups.find(key="Foo Device 2"),
+            preset,
+            self.mapping_parser,
+        )
         self.injector.stop_injecting()
         self.injector.run()
 
@@ -360,13 +375,13 @@ class TestInjector(unittest.IsolatedAsyncioTestCase):
         numlock_before = is_numlock_on()
 
         # stuff the preset outputs
-        system_mapping.clear()
+        keyboard_layout.clear()
         code_a = 100
         code_q = 101
         code_w = 102
-        system_mapping._set("a", code_a)
-        system_mapping._set("key_q", code_q)
-        system_mapping._set("w", code_w)
+        keyboard_layout._set("a", code_a)
+        keyboard_layout._set("key_q", code_q)
+        keyboard_layout._set("w", code_w)
 
         preset = Preset()
         preset.add(
@@ -405,7 +420,7 @@ class TestInjector(unittest.IsolatedAsyncioTestCase):
                 "a",
             )
         )
-        # one mapping that is unknown in the system_mapping on purpose
+        # one mapping that is unknown in the keyboard_layout on purpose
         input_b = 10
         with self.assertRaises(ValidationError):
             preset.add(
@@ -424,7 +439,11 @@ class TestInjector(unittest.IsolatedAsyncioTestCase):
                 )
             )
 
-        self.injector = Injector(groups.find(key="Foo Device 2"), preset)
+        self.injector = Injector(
+            groups.find(key="Foo Device 2"),
+            preset,
+            self.mapping_parser,
+        )
         self.assertEqual(self.injector.get_state(), InjectorState.UNKNOWN)
         self.injector.start()
         self.assertEqual(self.injector.get_state(), InjectorState.STARTING)
@@ -438,9 +457,10 @@ class TestInjector(unittest.IsolatedAsyncioTestCase):
             [
                 # should execute a macro...
                 InputEvent.key(8, 1),  # forwarded
-                InputEvent.key(9, 1),  # triggers macro
-                InputEvent.key(8, 0),  # releases macro
-                InputEvent.key(9, 0),  # forwarded
+                InputEvent.key(9, 1),  # triggers macro, not forwarding
+                # macro runs now and injects a few more keys
+                InputEvent.key(8, 0),  # releases macro (needs to be forwarded as well)
+                InputEvent.key(9, 0),  # not forwarded, just like the down-event
             ],
         )
 
@@ -477,20 +497,15 @@ class TestInjector(unittest.IsolatedAsyncioTestCase):
         history = read_write_history_pipe()
 
         # 1 event before the combination was triggered
-        # 2 events for releasing the combination trigger (by combination handler)
         # 4 events for the macro
-        # 1 release of the event that didn't release the macro
+        # 1 event for releasing the previous key-down event
         # 2 for mapped keys
         # 3 for forwarded events
-        self.assertEqual(len(history), 13)
+        self.assertEqual(len(history), 11)
 
         # the first bit is ordered properly
         self.assertEqual(history[0], (EV_KEY, 8, 1))  # forwarded
         del history[0]
-        self.assertIn((EV_KEY, 8, 0), history[0:2])  # released by combination handler
-        self.assertIn((EV_KEY, 9, 0), history[0:2])  # released by combination handler
-        del history[0]
-        del history[0]
 
         # since the macro takes a little bit of time to execute, its
         # keystrokes are all over the place.
@@ -512,10 +527,12 @@ class TestInjector(unittest.IsolatedAsyncioTestCase):
         del history[index_q_0]
         del history[index_q_1]
 
-        # the rest should be in order now.
-        # first the released combination key which did not release the macro.
-        # the combination key which released the macro won't appear here.
-        self.assertEqual(history[0], (EV_KEY, 9, 0))
+        # The rest should be in order now.
+        # First the released combination key which did not release the macro.
+        # The combination key which released the macro won't appear here, because
+        # it also didn't have a key-down event and therefore doesn't need to be
+        # released itself.
+        self.assertEqual(history[0], (EV_KEY, 8, 0))
         # value should be 1, even if the input event was -1.
         # Injected keycodes should always be either 0 or 1
         self.assertEqual(history[1], (EV_KEY, code_a, 1))
@@ -548,12 +565,13 @@ class TestInjector(unittest.IsolatedAsyncioTestCase):
         self.assertTrue(is_in_capabilities(key, capabilities))
 
 
+@test_setup
 class TestModifyCapabilities(unittest.TestCase):
-    @classmethod
-    def setUpClass(cls):
-        quick_cleanup()
-
     def setUp(self):
+        self.global_uinputs = GlobalUInputs(UInput)
+        self.global_uinputs.prepare_all()
+        self.mapping_parser = MappingParser(self.global_uinputs)
+
         class FakeDevice:
             def __init__(self):
                 self._capabilities = {
@@ -607,7 +625,7 @@ class TestModifyCapabilities(unittest.TestCase):
         )
 
         macro_code = "r(2, m(sHiFt_l, r(2, k(1).k(2))))"
-        macro = parse(macro_code, preset)
+        macro = Parser.parse(macro_code, preset)
 
         preset.add(
             Mapping.from_combination(
@@ -629,11 +647,11 @@ class TestModifyCapabilities(unittest.TestCase):
             ),
         )
 
-        self.a = system_mapping.get("a")
-        self.shift_l = system_mapping.get("ShIfT_L")
-        self.one = system_mapping.get(1)
-        self.two = system_mapping.get("2")
-        self.left = system_mapping.get("BtN_lEfT")
+        self.a = keyboard_layout.get("a")
+        self.shift_l = keyboard_layout.get("ShIfT_L")
+        self.one = keyboard_layout.get(1)
+        self.two = keyboard_layout.get("2")
+        self.left = keyboard_layout.get("BtN_lEfT")
         self.fake_device = FakeDevice()
         self.preset = preset
         self.macro = macro
@@ -648,13 +666,10 @@ class TestModifyCapabilities(unittest.TestCase):
         self.assertIn(self.shift_l, keys)
         self.assertNotIn(DISABLE_CODE, keys)
 
-    def tearDown(self):
-        quick_cleanup()
-
     def test_copy_capabilities(self):
         # I don't know what ABS_VOLUME is, for now I would like to just always
         # remove it until somebody complains, since its presence broke stuff
-        self.injector = Injector(mock.Mock(), self.preset)
+        self.injector = Injector(mock.Mock(), self.preset, self.mapping_parser)
         self.fake_device._capabilities = {
             EV_ABS: [ABS_VOLUME, (ABS_X, evdev.AbsInfo(0, 0, 500, 0, 0, 0))],
             EV_KEY: [1, 2, 3],
