@@ -1,7 +1,7 @@
-#!/usr/bin/python3
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 # input-remapper - GUI for device specific keyboard mappings
-# Copyright (C) 2023 sezanzeb <proxima@sezanzeb.de>
+# Copyright (C) 2025 sezanzeb <b8x45ygc9@mozmail.com>
 #
 # This file is part of input-remapper.
 #
@@ -23,8 +23,10 @@ import unittest
 import evdev
 from dataclasses import FrozenInstanceError
 from inputremapper.input_event import InputEvent
+from tests.lib.test_setup import test_setup
 
 
+@test_setup
 class TestInputEvent(unittest.TestCase):
     def test_from_event(self):
         e1 = InputEvent.from_event(evdev.InputEvent(1, 2, 3, 4, 5))
